# /packages/intranet-budget/www/action.tcl
#
# Copyright (C) 2003-2008 ]project-open[
#
# All rights reserved. Please check
# http://www.project-open.com/license/ for details.

ad_page_contract {
    Perform bulk actions on budget_items
    
    @action_id	One of "Intranet Conf_Item Action" categories.
    		Determines what to do with the list of "tid"
		budget_item ids.
		The "aux_string1" field of the category determines
		the page to be called for pluggable actions.

    @param return_url the url to return to
    @author frank.bergmann@project-open.com
} {
    budget_item_object_id:integer
    { budget_item_id:array ""}
    action_id:integer
    return_url
}

set user_id [auth::require_login]
set user_is_admin_p [im_is_user_site_wide_or_intranet_admin $user_id]
set user_name [im_name_from_user_id [ad_conn user_id]]

# Deal with funky input parameter combinations
if {"" == $action_id} { ad_returnredirect $return_url }
# if {0 == [llength $budget_item]} { ad_returnredirect $return_url }
# if {1 == [llength $budget_item]} { set budget_item [lindex $budget_item 0] }
# ad_return_complaint 1 "$action_id - [array get budget_item_id]"


set action_name [im_category_from_id $action_id]
set action_forbidden_msg [lang::message::lookup "" intranet-budget.Action_Forbidden "<b>Unable to execute action</b>:<br>You don't have the permissions to execute the action '%action_name%' on this budget_item."]

# ------------------------------------------
# Check the TCL that determines the visibility of the action
set visible_tcl [util_memoize [list db_string visible_tcl "select visible_tcl from im_categories where category_id = $action_id"]]
set visible_p 0
set visible_explicite_permission_p 0
if {"" == $visible_tcl} {
    # Not specified - User is allowed to execute but normal permissions apply
    set visible_p 1
} else {
    # Explicitely specified: Check TCL
    if {[eval $visible_tcl]} {
	set visible_p 1
	set visible_explicite_permission_p 1
    }
}

# ------------------------------------------
# Perform the action on multiple budget_items
#
switch $action_id {
    49200 {
	# Save
	set hard_attributes {
	    budget_item_id 
	    budget_item_name budget_item_nr budget_item_parent_id 
	    budget_item_type_id budget_item_status_id 
	    budget_item_owner_id budget_item_object_id 
	    sort_order description note 
	    tree_sortkey max_child_sortkey 
	    match_cost_type_id match_expense_type_id match_task_id match_material_type_id 
	    budget_item_max_value budget_item_alarm_value 
	    budget_item_current_value_cache budget_item_current_value_cache_date
        }
	set dynfield_attributes [db_list attrs "
		select	attribute_name
		from	acs_attributes aa,
			im_dynfield_attributes da
		where	da.acs_attribute_id = aa.attribute_id and
			aa.object_type = 'im_budget_item'
		order by aa.attribute_name
        "]
	set attributes [lsort -unique [concat $hard_attributes $dynfield_attributes]]


	set query [ns_conn query]
	set query_set [ns_parsequery $query]
	array set query_hash [ns_set array $query_set]

	# We're going to collect the list of budget_item_ids and the list of variables
	array set item_id_hash {}
	array set var_hash {}
	foreach var [array names query_hash] {
	    set var_pieces [split $var "."]
	    set var_name [lindex $var_pieces 0]
	    set item_id [lindex $var_pieces 1]
	    if {"" eq $item_id || "" eq $var_name} { continue }
	    if {[lsearch $attributes $var_name] < 0} { continue }

	    set item_id_hash($item_id) $item_id
	    set var_hash($var_name) $var_name
	}
	# ad_return_complaint 1 "[array get item_id_hash]<br>[array get var_hash]"
	# ad_return_complaint 1 "[array get query_hash]<br><br>$attributes"

	foreach item_id [array names item_id_hash] {
	    set update_list [list]
	    foreach var [array names var_hash] {
		set varname "$var.$item_id"
		set value ""
		if {[info exists query_hash($varname)]} { set value $query_hash($varname) }
		if {"" eq $value} { set value NULL } else { set value "'$value'" }
		lappend update_list "$var = $value"
	    }
	    db_dml update "update im_budget_items set [join $update_list ",\n"] where budget_item_id = :item_id"
	}
    }
    49290 {
	# Delete
	if {!$user_is_admin_p} { 
	    ad_return_complaint 1 "User needs to be SysAdmin in order to 'Nuke' budget_items." 
	    ad_script_abort
	}
	set error_list [list]
	foreach cid [array names budget_item_id] {
	    im_budget_item::check_permissions	-budget_item_id $cid -operation "admin"
	    im_budget_item::audit		-budget_item_id $cid -action "before_nuke"

	    if {[catch {
		im_budget_item::nuke		-budget_item_id $cid
	    } err_msg]} {
		lappend error_list $err_msg
	    }
	}

	if {[llength $error_list] > 0} {
	    ad_return_complaint 1 "<b>Errors while nuking budget items</b>:<br><pre>[join $error_list "<br>"]</pre>"
	    ad_script_abort
	}
	
	# Conf_Item may not exist anymore, return to budget_item list
	if {[regexp {^\/intranet-budget\/new} $return_url match]} {
	    set return_url "/intranet-budget/"
	}
    }
    12600 - 12602 - 12616 - 12618 {
	switch $action_id {
	    12600 { set status_id 11700 }
	    12602 { set status_id 11702 }
	    12616 { set status_id 11716 }
	    12618 { set status_id 11718 }
	    default { set status_id "" }
	}
	if {"" eq $status_id} { ad_returnredirect $return_url }

	# Set a specific status
	foreach cid $budget_item {
	    im_budget_item::audit			-budget_item_id $cid -action "before_update"
	    if {!$visible_explicite_permission_p} {
		im_budget_item::check_permissions	-budget_item_id $cid -operation "write"
	    }
	    im_budget_item::set_status_id		-budget_item_id $cid -budget_item_status_id $status_id
	    im_budget_item::audit			-budget_item_id $cid -action "after_update"
	}
	
	if {$action_id == 30510} {
	    # Close & Notify - Notify all stakeholders
	    ad_returnredirect [export_vars -base "/intranet-budget/notify-stakeholders" {budget_item action_id return_url}]
	}
    }
    12620 {
	# Associate
	ad_return_complaint 1 "Not implemented yet"
	ad_returnredirect [export_vars -base "/intranet-budget/action-associate" {budget_item action_id return_url}]
    }
    default {
	# Check if we've got a custom action to perform
	set redirect_base_url [db_string redir "select aux_string1 from im_categories where category_id = :action_id" -default ""]
	if {"" != [string trim $redirect_base_url]} {
	    # Redirect for custom action
	    set redirect_url [export_vars -base $redirect_base_url {action_id return_url}]
	    foreach cid $budget_item { append redirect_url "&budget_item=$cid"}
	    ad_returnredirect $redirect_url
	} else {
	    ad_return_complaint 1 "Unknown Conf_Item action: $action_id='[im_category_from_id $action_id]'"
	}
    }
}


ad_returnredirect $return_url
