# /packages/intranet-budget/www/new.tcl
#
# Copyright (c) 2003-2007 ]project-open[
#
# All rights reserved. Please check
# http://www.project-open.com/license/ for details.

# ---------------------------------------------------------------
# Page Contract
# ---------------------------------------------------------------

ad_page_contract {
    Show or create a new budget item
    @author frank.bergmann@project-open.com
    @parameter view_name Set to "component" in order to show a specific component

    @parameter budget_item_project_id Project or Cost Center to which the budget item 
               relates
} {
    budget_item_id:integer,optional
    { return_url ""}
    { form_mode "edit" }
    { view_name "" }
    budget_item_type_id:integer,optional
    { budget_item_project_id "" }
    { enable_master_p 1}
}

set current_user_id [auth::require_login]
set current_url [im_url_with_query]
set user_admin_p [im_is_user_site_wide_or_intranet_admin $current_user_id]
set focus ""
set sub_navbar ""

# org_budget_item_id required by Portlet Components!
set org_budget_item_id [im_opt_val -limit_to integer budget_item_id]

set page_title [lang::message::lookup "" intranet-budget.New_Budget_Item "New Budget Item"]
set show_components_p 0
if {"display" == $form_mode || "" == $form_mode} {
    set page_title [lang::message::lookup "" intranet-budget.Budget_Item "Budget Item"]
    set show_components_p 1

    # Write Audit Trail
    if {[info exists budget_item_id]} {
	im_audit -object_type "im_budget_item" -object_id $budget_item_id -action before_view
    }
}
set context_bar [im_context_bar $page_title]

if {"" == $return_url} {
    switch $form_mode {
	display { set return_url [im_url_with_query] }
	edit { set return_url "/intranet-budget/index" }
    }
}

# ---------------------------------------------------------------
# Permissions
# ---------------------------------------------------------------

set budget_item_exists_p 0
if {[info exists budget_item_id] && "" != $budget_item_id && 0 != $budget_item_id} {
    set budget_item_exists_p [db_string ex "select count(*) from im_budget_items where budget_item_id = :budget_item_id"]
}

if {$budget_item_exists_p} {

    # The budget item exists - check for read or write, depending on mode
    set budget_item_type_id [db_string tid "select budget_item_type_id from im_budget_items where budget_item_id = :budget_item_id" -default ""]
    im_budget_item_permissions $current_user_id $budget_item_id view_p read_p write_p admin_p
    if {$admin_p} { set user_admin_p 1 }

    switch $form_mode {
	"edit" {
	    if {!$write_p} {
		ad_return_complaint 1 "You don't have sufficient permissions to modify this budget item"
		ad_script_abort
	    }
	}
	"display" {
	    if {!$read_p} {
		ad_return_complaint 1 "You don't have sufficient permissions to see this budget item"
		ad_script_abort
	    }
	}
	default {
	    ad_return_complaint 1 "Unknown form_mode='$form_mode'"
	    ad_script_abort
	}
    }

} else {

    # No budget_item_id is set - so the user tries to create a new item
    switch $form_mode {
	"edit" {
	    if {![im_permission $current_user_id "add_budget_items"]} {
		ad_return_complaint 1 "You don't have sufficient permissions to create or modify budget items"
		ad_script_abort
	    }
	}
	default {
	    ad_return_complaint 1 "Unknown form_mode='$form_mode'"
	    ad_script_abort
	}
    }
}




# ------------------------------------------------------------------
# Redirect if budget_item_type_id is missing
# ------------------------------------------------------------------

if {![info exists budget_item_type_id] || "" eq $budget_item_type_id || 0 == $budget_item_type_id} {
    set all_same_p [im_dynfield::subtype_have_same_attributes_p -object_type "im_budget_item"]
    if {!$all_same_p && "edit" == $form_mode} {
	set new_typeselect_url_default "/intranet-budget/new-typeselect"
	set new_typeselect_url [parameter::get_from_package_key -package_key "intranet-budget" -parameter "NewTypeSelectUrl" -default $new_typeselect_url_default]
	if {"" == $new_typeselect_url} { set new_typeselect_url $new_typeselect_url_default }
	ad_returnredirect [export_vars -base $new_typeselect_url {{return_url $current_url} budget_item_id budget_item_type_id budget_item_name budget_item_nr}]
	
    }
}


if {![info exists budget_item_type_id]} {
    set budget_item_type_id [im_budget_item_type_default]
}


# ---------------------------------------------------------------
# 
# ---------------------------------------------------------------

# Options - get the value range for input fields
set budget_item_parent_options [im_budget_item_options -include_empty_p 1]
set project_options [util_memoize [list im_project_options -include_empty 1 -project_status_id [im_project_status_open]] 3600]
set owner_options [util_memoize [list im_user_options]]

set action_url "/intranet-budget/new"


set form_id "budget_item"

ad_form \
    -name $form_id \
    -mode $form_mode \
    -action $action_url \
    -export return_url \
    -form {
	budget_item_id:key
	{budget_item_name:text(text),optional {label "[lang::message::lookup {} intranet-budget.Budget_Item_Name {Budget Item Name}]"} {html {size 60}}}
	{budget_item_nr:text(text) {label "[lang::message::lookup {} intranet-budget.Budget_Item_Nr {Budget Item Nr.}]"} {html {size 40}}}
    	{budget_item_project_id:text(select),optional {
		label "[lang::message::lookup {} intranet-budget.Related_Object {Related Object}]"
	} {options $project_options }}
	{budget_item_parent_id:text(select),optional {label "[lang::message::lookup {} intranet-budget.Budget_Item_Parent {Budget Item Parent}]"} {options $budget_item_parent_options} }
	{budget_item_type_id:text(im_category_tree) {label "[lang::message::lookup {} intranet-budget.Budget_Item_Type {Type}]"} {custom {category_type "Intranet Budget Item Type" translate_p 1 include_empty_p 0} } }
	{budget_item_status_id:text(im_category_tree) {label "[lang::message::lookup {} intranet-budget.Budget_Item_Status {Status}]"} {custom {category_type "Intranet Budget Item Status" translate_p 1 include_empty_p 0}} }
    	{budget_item_owner_id:text(select),optional {label "[lang::message::lookup {} intranet-budget.Owner_Manager {Owner / Manager}]"} {options $owner_options }}

	{budget_item_max_value:float(text),optional {label "[lang::message::lookup {} intranet-budget.Budget_Item_Max_Value {Maximum Value}]"} }
	{budget_item_alarm_value:float(text),optional {label "[lang::message::lookup {} intranet-budget.Budget_Item_Alarm_Value {Alarm Value}]"} }

	{description:text(textarea),optional {label Description} {html {cols 40} {rows 8} }}
	{note:text(textarea),optional {label Note} {html {cols 40} {rows 8} }}
    }

# Pre-set budget_item_project_id
if { [info exists budget_item_project_id] && "" != $budget_item_project_id } {
	ad_set_element_value -element budget_item_project_id $budget_item_project_id
}

# Check that the Intranet Budget Item Type exists or set to default.
if {![info exists budget_item_type_id]} { 
    set budget_item_type_id ""
    if {[info exists budget_item_id]} {
	set budget_item_type_id [db_string budget_item_type "select budget_item_type_id from im_budget_items where budget_item_id = :budget_item_id" -default ""]
    }
}

# Add DynField attributes.
if {[info exists budget_item_type_id]} { 

    template::element::set_value $form_id budget_item_type_id $budget_item_type_id
    set field_cnt [im_dynfield::append_attributes_to_form \
		       -form_display_mode $form_mode \
		       -object_subtype_id $budget_item_type_id \
		       -object_type "im_budget_item" \
		       -form_id $form_id
    ]
}


ad_form -extend -name $form_id \
    -select_query {
	select	*
	from	im_budget_items
	where	budget_item_id = :budget_item_id
    } -new_data {
	if {![info exists budget_item_name] || "" == $budget_item_name} {
	    set budget_item_name $budget_item_nr
	}
	set budget_item_new_sql "
		select im_budget_item__new(
			:budget_item_id, 'im_budget_item', now(), :current_user_id, '[ad_conn peeraddr]', null,
			:budget_item_name,
			:budget_item_nr,
			:budget_item_parent_id,
			:budget_item_type_id,
			:budget_item_status_id,
			:budget_item_project_id
		)
	"
	set exists_p [db_string exists "select count(*) from im_budget_items where budget_item_id = :budget_item_id"]
	if {!$exists_p} { set budget_item_id [db_string new $budget_item_new_sql] }
	db_dml update [im_budget_item_update_sql -include_dynfields_p 1]

	# Store DynFields
	im_dynfield::attribute_store \
	    -object_type "im_budget_item" \
	    -object_id $budget_item_id \
	    -form_id $form_id

	# Establish a im_budget_item_projects_rel
	set rel_id [db_string rel "select im_budget_item_project_rel__new (null, 'im_budget_item_project_rel', :budget_item_project_id, :budget_item_id, null, :current_user_id, '[ad_conn peeraddr]', 0)"]

	# Write an audit record 
	im_audit -object_type "im_budget_item" -object_id $budget_item_id -action after_create

    } -edit_data {

	# Write an audit record _before_ the update, in case the budget item
	# was modified outside of ]po[ (ugly, but may happen...)
	im_audit -object_type "im_budget_item" -object_id $budget_item_id -action before_update

	if {![info exists budget_item_name] || "" == $budget_item_name} {
	    set budget_item_name $budget_item_nr
	}
	db_dml update [im_budget_item_update_sql -include_dynfields_p 1]

	im_dynfield::attribute_store \
	    -object_type "im_budget_item" \
	    -object_id $budget_item_id \
	    -form_id $form_id

	# Write an audit record 
	im_audit -object_type "im_budget_item" -object_id $budget_item_id -action after_update


    } -after_submit {
	ad_returnredirect $return_url
	ad_script_abort
    }


# ---------------------------------------------------------------
# List of Sub-Items
# ---------------------------------------------------------------

set export_var_list [list]
set bulk_actions_list [list]
set delete_budget_item_p 1
if {$delete_budget_item_p} {
    lappend bulk_actions_list "[lang::message::lookup "" intranet-budget.Delete "Delete"]" "budget-item-del" "[lang::message::lookup "" intranet-budget.Remove_checked_items "Remove Checked Items"]"
}

template::list::create \
    -name sub_budget_items \
    -multirow budget_item_lines \
    -key budget_item_id \
    -has_checkboxes \
    -bulk_actions $bulk_actions_list \
    -bulk_action_export_vars { return_url} \
    -row_pretty_plural "[lang::message::lookup "" intranet-budget.Budget_Items_Items {Budget Items}]" \
    -elements {
	budget_item_chk {
	    label "<input type=\"checkbox\" 
			  name=\"_dummy\" 
			  onclick=\"acs_ListCheckAll('budget_items_list', this.checked)\" 
			  title=\"Check/uncheck all rows\">"
	    display_template {
		@budget_item_lines.budget_item_chk;noquote@
	    }
	}
	budget_item_name {
	    label "[lang::message::lookup {} intranet-budget.Budget_Item_Name Name]"
	    display_template {
		@budget_item_lines.indent;noquote@<a href=@budget_item_lines.budget_item_url;noquote@>@budget_item_lines.budget_item_name;noquote@</a>
	    }
	}
	budget_item_status {
	    label "[lang::message::lookup {} intranet-budget.Budget_Item_Status Status]"
	}
    }

set budget_item_sql [im_budget_item_select_sql \
	-object_id "" \
	-type_id "" \
	-status_id "" \
	-owner_id "" \
	-treelevel "" \
	-parent_id [im_opt_val -limit_to integer budget_item_id] \
]

set sql "
	select	i.*,
		tree_level(i.tree_sortkey)-1 as indent_level,
		p.project_id,
		project_name
	from	($budget_item_sql) i
		LEFT OUTER JOIN acs_rels r ON (i.budget_item_id = r.object_id_two)
		LEFT OUTER JOIN im_projects p ON (p.project_id = r.object_id_one)
	order by
		i.tree_sortkey
"

# ad_return_complaint 1 "<pre>$sql</pre>"

set sub_item_count 0
db_multirow -extend {budget_item_chk budget_item_url indent return_url} budget_item_lines budget_items_lines $sql {
    incr sub_item_count
    set budget_item_chk "<input type=\"checkbox\" 
				name=\"budget_item_id\" 
				value=\"$budget_item_id\" 
				id=\"budget_items_list,$budget_item_id\">"
    set return_url [im_url_with_query]
    set budget_item_url [export_vars -base new {budget_item_id {form_mode "display"}}]

    set indent ""
    for {set i 0} {$i < $indent_level} {incr i} {
	append indent "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
    }
}


# budget_item_id overwritten by query of sub-elements...
set budget_item_id $org_budget_item_id



# ---------------------------------------------------------------
# Show dumb tables
# ---------------------------------------------------------------

set result ""
