# /packages/intranet-budget/www/version/index.tcl
#
# Copyright (C) 2003 - 2019 ]project-open[
#
# All rights reserved. Please check
# http://www.project-open.com/license/ for details.

# ---------------------------------------------------------------
# Page Contract
# ---------------------------------------------------------------

ad_page_contract { 
    @author frank.bergmann@project-open.com
} {
    { object_id 0}
}

# ---------------------------------------------------------------
# Defaults & Security
# ---------------------------------------------------------------

# User id already verified by filters
set user_id [auth::require_login]
set current_user_id $user_id
set page_focus "im_header_form.keywords"
set user_admin_p [im_is_user_site_wide_or_intranet_admin $current_user_id]
set date_format "YYYY-MM-DD"
set object_name [db_string object_name "select acs_object__name(:object_id)" -default [lang::message::lookup "" intranet-core.Unassigned "Unassigned"]]
set page_title [lang::message::lookup "" intranet-budget.Budget_Versions "Budget Versions"]
set context_bar [im_context_bar $page_title]
set return_url [im_url_with_query]
set current_url [ns_conn url]
set edit_project_all_p [im_permission $current_user_id "edit_projects_all"]


# ---------------------------------------------------------------
# Admin Links
# ---------------------------------------------------------------

set admin_links ""
append admin_links " <li><a href=\"[export_vars -base new {object_id return_url}]\">[lang::message::lookup "" intranet-budget.Add_a_new_Budget_Version "Add a new budget version"]</a>\n"
if {"" != $admin_links} { set admin_links "<ul>\n$admin_links</ul>\n" }

set bulk_actions_list "[list]"
set delete_expense_p 1 
if {$delete_expense_p} {
    lappend bulk_actions_list "[_ intranet-core.Delete]" "delete" "[_ intranet-core.Remove_checked_items]"
}

set create_invoice_p 1
if {$create_invoice_p} {
}

# ---------------------------------------------------------------
#
# ---------------------------------------------------------------

# Variables of this page to pass through the expenses_page

set export_var_list [list]

# define list object
set list_id "versions_list"




set actions [list \
		 [lang::message::lookup "" intranet-budget.New_Budget_Version "New Budget Version"] \
		 [export_vars -base "/intranet-budget/versions/new"] \
		 [lang::message::lookup "" intranet-budget.Create_new_version_help_blurb "Create a new budget version"] \
]

set actions {}

template::list::create \
    -name $list_id \
    -multirow version_lines \
    -key version_id \
    -has_checkboxes \
    -actions $actions \
    -bulk_action_method GET \
    -bulk_actions $bulk_actions_list \
    -bulk_action_export_vars  {
	object_id
	return_url
    } \
    -row_pretty_plural "[_ intranet-budget.Versions_Items]" \
    -elements {
	version_chk {
	    label "<input type=\"checkbox\" 
                          name=\"_dummy\" 
                          onclick=\"acs_ListCheckAll('versions_list', this.checked)\" 
                          title=\"Check/uncheck all rows\">"
	    display_template {
		@version_lines.version_chk;noquote@
	    }
	}
	version_name {
	    label "[lang::message::lookup {} intranet-budget.Version_name {Version Name}]"
	    link_url_eval {[export_vars -base "/intranet-budget/versions/new" {{form_mode "display"} version_id}]}
	}
	version_nr {
	    label "[lang::message::lookup {} intranet-budget.Version_nr {Version Nr}]"
	    link_url_eval {[export_vars -base "/intranet-budget/versions/new" {{form_mode "display"} version_id}]}
	}
	object_name {
	    label "[lang::message::lookup {} intranet-budget.Object Object]"
	    link_url_eval {[export_vars -base $object_url {{object_id $version_object_id}}]}
	}
        version_type {
	    label "[lang::message::lookup {} intranet-budget.Version_Type Type]"
	}
        version_status {
	    label "[lang::message::lookup {} intranet-budget.Version_Status Status]"
	}
    }

# Only show version_items of objects that are controlled by the user
set project_permission_where "1=1"
if {![im_permission $current_user_id "view_projects_all"]} {
    set project_permission_where "
	p.project_id in (
		select	r.object_id_one
		from	acs_rels r
		where	r.object_id_two = :current_user_id
	)
    "
}


db_multirow -extend {version_chk object_name} version_lines versions_lines "
	select	bv.*,
		o.object_id,
		bou.url as object_url,
		im_category_from_id(bv.version_status_id) as version_status,
		im_category_from_id(bv.version_type_id) as version_type,
		(select count(*) 
		from	acs_rels r,
			im_biz_object_members bom
		where	r.rel_id = bom.rel_id and
			bom.object_role_id in (1301,1302,1303,1309) and
			r.object_id_two = :current_user_id and 
			r.object_id_one = bv.version_object_id
		) as write_p
	from	im_budget_versions bv
		LEFT OUTER JOIN acs_objects o ON (bv.version_object_id = o.object_id)
		LEFT OUTER JOIN im_biz_object_urls bou ON (bou.object_type = o.object_type and bou.url_type = 'view')
	where	$project_permission_where
" {
    set version_chk "<input type=\"checkbox\" name=\"version_id\" value=\"$version_id\" id=\"versions_list,$version_id\">"
    if {!$edit_project_all_p && !$write_p} { set version_chk "" }
    set object_name $object_name
}

