# /packages/intranet-budget/www/versions/new.tcl
#
# Copyright (C) 2003-2006 ]project-open[
# all@devcon.project-open.com
#
# All rights reserved. Please check
# http://www.project-open.com/license/ for details.

# ---------------------------------------------------------------
# Page Contract
# ---------------------------------------------------------------

# Use the page contract only if called as a normal page...
# As a panel, the calling script needs to provide the necessary
# variables.
if {![info exists panel_p]} {
    ad_page_contract {
	New page is basic...
	@author all@devcon.project-open.com
    } {
	version_id:integer,optional
	{return_url "/intranet-budget/versions/index"}
	form_mode:optional
	enable_master_p:integer,optional
    }
}


set user_id [auth::require_login]
set page_title [lang::message::lookup "" intranet-budget.Budget_Version "Budget Version"]
set context_bar [im_context_bar $page_title]

if {![info exists enable_master_p]} { set enable_master_p 1}
if {![info exists form_mode]} { set form_mode "display" }
if {![info exists message]} { set message "" }

# ---------------------------------------------------------------
# Options
# ---------------------------------------------------------------

set version_object_options [im_project_options]
set version_object_nr ""

if {[info exists version_id]} {
    # Check if the version_objbect exists
    set version_object_exists_p [db_string exists "select count(*) from im_budget_versions where version_id = :version_id"]

    # Add the version_item's project to the options, if not already there
    # Otherwise the component can't show the project's name
    set version_object_id [db_string version_pid "select version_object_id from im_budget_versions where version_id = :version_id" -default ""]

    set found_p 0
    foreach ptuple $version_object_options {
	set pid [lindex $ptuple 1]
	if {$pid == $version_object_id} { set found_p 1 }
    }
    if {!$found_p} {
	set version_object_name [db_string version_pid "select project_name from im_projects where project_id = :version_object_id" -default ""]
	lappend version_object_options [list $version_object_name $version_object_id]
    }
    set version_object_nr [db_string version_pid "select project_nr from im_projects where project_id = :version_object_id" -default ""]
}

set version_type_options [db_list_of_lists version_type_options "
	select	version_type, version_type_id
	from	im_budget_version_type
	order by version_type_id
"]
set version_status_options [db_list_of_lists version_status_options "
	select	version_status, version_status_id
	from	im_budget_version_status
	order by version_status_id
"]


# ------------------------------------------------------------------
# Actions & Their Permissions
# ------------------------------------------------------------------

set actions [list]

if {[info exists version_id]} {

    if {![ad_form_new_p -key version_id]} {
	set version_exists_p [db_string count "select count(*) from im_budget_versions where version_id=:version_id" -default 0]
	if {!$version_exists_p} {
	    ad_return_complaint 1 "<b>Error: The selected version (#$version_id) does not exist</b>:<br>
	The object has probably been deleted by its owner recently."
	    ad_script_abort
	}
    }

    set ttt {
	set edit_perm_func [parameter::get_from_package_key -package_key intranet-budget -parameter BudgetVersionNewPageWfEditButtonPerm -default "im_budget_version_new_page_wf_perm_edit_button"]
	set delete_perm_func [parameter::get_from_package_key -package_key intranet-budget -parameter BudgetVersionNewPageWfDeleteButtonPerm -default "im_budget_version_new_page_wf_perm_delete_button"]
	
	if {[eval [list $edit_perm_func -version_id $version_id]]} {
    lappend actions [list [lang::message::lookup {} intranet-budget.Edit Edit] edit]
	}
	if {[eval [list $delete_perm_func -version_id $version_id]]} {
    lappend actions [list [lang::message::lookup {} intranet-budget.Delete Delete] delete]
	}
    }


}


# ------------------------------------------------------------------
# Delete pressed?
# ------------------------------------------------------------------

set button_pressed [template::form get_action form]
if {"delete" == $button_pressed} {
    db_string version_delete "select im_budget_version__delete(:version_id)"

    # In some cases, return_url is set to the page showing the case that had been just deleted
    if { [string first "/acs-workflow/case?case_id" [string tolower $return_url]] == 0 } {
	ad_returnredirect "/intranet-budget/versions/"
    } else {
	ad_returnredirect $return_url
    }
}

# ---------------------------------------------------------------
# The Form
# ---------------------------------------------------------------

set form_id "form"
ad_form \
    -name $form_id \
    -mode $form_mode \
    -export "return_url" \
    -actions $actions \
    -action "/intranet-budget/versions/new" \
    -form {
	version_id:key
	{version_name:text(text) {label "[lang::message::lookup {} intranet-budget.Version_Name {Version Name}]"} {html {size 50}} }
	{version_nr:text(text) {label "[lang::message::lookup {} intranet-budget.Version_Nr {Version Nr}]"} {html {size 30}} }
	{version_object_id:text(select),optional {label "[lang::message::lookup {} intranet-budget.Version_Object Object]"} {options $version_object_options} }
	{version_type_id:text(select) {label "[lang::message::lookup {} intranet-budget.Version_Type Type]"} {options $version_type_options} }
	{version_status_id:text(select) {label "[lang::message::lookup {} intranet-budget.Version_Status Status]"} {options $version_status_options} }
    }

ad_form -extend -name $form_id \
    -select_query {
	select	*
	from	im_budget_versions
	where	version_id = :version_id
    } -new_data {
	db_exec_plsql create_version "
		SELECT im_budget_version__new(
			:version_id, 'im_version', now(), :user_id, '[ad_conn peeraddr]', null,
			:version_name, version_nr,
			:version_object_id,
			:version_type_id,
			:version_status_id
		)
        "
    } -edit_data {
	db_dml edit_version "
		update im_budget_versions set 
			version_name = :version_name,
			version_nr = :version_nr,
			version_object_id = :version_object_id,
			version_type_id = :version_type_id,
			version_status_id = :version_status_id
		where version_id = :version_id
	"
    } -after_submit {
	ad_returnredirect $return_url
	ad_script_abort
    }


# Don't show the portlets on the page...
set form_mode none

